#! /usr/bin/env python3

from os import path

from PyFoam.Applications.RedoPlot import RedoPlot
from PyFoam.RunDictionary.TimelineDirectory import TimelineDirectory
from PyFoam.RunDictionary.ParsedParameterFile import ParsedParameterFile

from scipy.integrate import odeint, solve_ivp
import numpy as np

def calcHeights(caseName):
    rhoWater = ParsedParameterFile(path.join(caseName, "constant", "thermophysicalProperties.water"))["mixture"]["equationOfState"]["rho0"]
    bottleParameters = ParsedParameterFile(path.join(caseName, "constant", "bottleParameters"))

    tl = TimelineDirectory(case=caseName,
                           dirName="postProcessing/swakExpression_acceleration")
    accel2 = tl["acceleration"]().getData()

    mainFrac = TimelineDirectory(case=caseName,
                                 dirName="postProcessing/swakExpression_mainPhaseFraction")
    mFrac = mainFrac["mainPhaseFraction"]().getData()

    totalMassData = TimelineDirectory(case=caseName,
                                      dirName="postProcessing/swakExpression_totalMass")
    totalMass = totalMassData["totalMass"]().getData()

    grav = 9.81

    def func(t, x):
        return [x[1],
                accel2[t]["acceleration_t=0 sum"].values[0]-grav]


    times = np.linspace(0, 10, 100000)

    sol = solve_ivp(func, [0, 10], [0, 0], t_eval=times)


    densAir = 1.15
    Cd = 0.82    # Long cylinder

    massBottle = bottleParameters["bottleWeightGram"]/1000.

    bottleRadius = bottleParameters["bottleRadiusCm"]/100.
    bottleHeight = bottleParameters["bottleHeightCm"]/100.

    area = bottleRadius*bottleRadius*3.1415
    vol = area*bottleHeight


    def funcDrag(t, x):
        # mass = vol*rhoWater*mFrac[t]["mainPhaseFraction_t=0 weightedAverage"].values[0] + massBottle
        mass = totalMass[t]["totalMass_t=0 max"].values[0]
        dragForce = 0.5*densAir*Cd*area*x[1]*abs(x[1])/mass
        velNoDrag, accelNoDrag = func(t,x)
        return [velNoDrag,
                accelNoDrag-dragForce]


    solDrag = solve_ivp(funcDrag, [0, 10], [0, 0], t_eval=times)

    return sol,solDrag

def solutionToDataFrame(sol):
    from PyFoam.Wrappers.Pandas import PyFoamDataFrame

    return PyFoamDataFrame(data={"h": sol.y[0], "vel": sol.y[1]},
                           index=sol.t)

if __name__ == "__main__":
    import sys
    if len(sys.argv) != 2:
        print("Script needs 1 argument: <case directory>")
        sys.exit(1)

    caseName = sys.argv[1]

    solNoDrag, solDrag = calcHeights(caseName)

    noDrag = solutionToDataFrame(solNoDrag)
    withDrag = solutionToDataFrame(solDrag)

    print("Max height without drag (solve_ivp) {} at t={}".format(noDrag.h.max(),noDrag.h.idxmax()))
    print("Max velocity without drag (solve_ivp) {} at t={} h={}".format(noDrag.vel.max(),noDrag.vel.idxmax(),noDrag.h[noDrag.vel.idxmax()]))
    print("Max height with drag (solve_ivp) {} at t={}".format(withDrag.h.max(),withDrag.h.idxmax()))
    print("Max velocity with drag (solve_ivp) {} at t={} h={}".format(withDrag.vel.max(),withDrag.vel.idxmax(),withDrag.h[withDrag.vel.idxmax()]))
